@@ -1,3 +1,5 @@
+from contextlib import contextmanager
+
 from dagster import check
 from dagster.core.definitions import ExecutablePipeline, PipelineDefinition, SystemStorageData
 from dagster.core.definitions.executable import InMemoryExecutablePipeline
@@ -9,11 +11,13 @@ from dagster.core.execution.plan.execute_plan import inner_plan_execution_iterat
 from dagster.core.execution.plan.plan import ExecutionPlan
 from dagster.core.execution.retries import Retries
 from dagster.core.instance import DagsterInstance
+from dagster.core.selector import parse_step_selection
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
 from dagster.core.system_config.objects import EnvironmentConfig
 from dagster.core.telemetry import log_repo_stats, telemetry_wrapper
 from dagster.core.utils import str_format_set
 from dagster.utils import merge_dicts
+from dagster.utils.backcompat import canonicalize_backcompat_args
 
 from .context_creation_pipeline import (
     ExecutionContextManager,
@@ -218,34 +222,42 @@ def execute_pipeline_iterator(
       Iterator[DagsterEvent]: The stream of events resulting from pipeline execution.
     """
 
-    (
-        pipeline,
-        run_config,
-        instance,
-        mode,
-        tags,
-        solids_to_execute,
-        solid_selection,
-    ) = _check_execute_pipeline_args(
-        pipeline=pipeline,
-        run_config=run_config,
-        mode=mode,
-        preset=preset,
-        tags=tags,
-        solid_selection=solid_selection,
-        instance=instance,
-    )
+    with _ephemeral_instance_if_missing(instance) as execute_instance:
+        (
+            pipeline,
+            run_config,
+            mode,
+            tags,
+            solids_to_execute,
+            solid_selection,
+        ) = _check_execute_pipeline_args(
+            pipeline=pipeline,
+            run_config=run_config,
+            mode=mode,
+            preset=preset,
+            tags=tags,
+            solid_selection=solid_selection,
+        )
 
-    pipeline_run = instance.create_run_for_pipeline(
-        pipeline_def=pipeline.get_definition(),
-        run_config=run_config,
-        mode=mode,
-        solid_selection=solid_selection,
-        solids_to_execute=solids_to_execute,
-        tags=tags,
-    )
+        pipeline_run = execute_instance.create_run_for_pipeline(
+            pipeline_def=pipeline.get_definition(),
+            run_config=run_config,
+            mode=mode,
+            solid_selection=solid_selection,
+            solids_to_execute=solids_to_execute,
+            tags=tags,
+        )
 
-    return execute_run_iterator(pipeline, pipeline_run, instance)
+        return execute_run_iterator(pipeline, pipeline_run, execute_instance)
+
+
+@contextmanager
+def _ephemeral_instance_if_missing(instance):
+    if instance:
+        yield instance
+    else:
+        with DagsterInstance.ephemeral() as ephemeral_instance:
+            yield ephemeral_instance
 
 
 def execute_pipeline(
@@ -294,35 +306,35 @@ def execute_pipeline(
     This is the entrypoint for dagster CLI execution. For the dagster-graphql entrypoint, see
     ``dagster.core.execution.api.execute_plan()``.
     """
-    instance = instance or DagsterInstance.ephemeral()
-    return _logged_execute_pipeline(
-        pipeline,
-        run_config=run_config,
-        mode=mode,
-        preset=preset,
-        tags=tags,
-        solid_selection=solid_selection,
-        instance=instance,
-        raise_on_error=raise_on_error,
-    )
+
+    with _ephemeral_instance_if_missing(instance) as execute_instance:
+        return _logged_execute_pipeline(
+            pipeline,
+            instance=execute_instance,
+            run_config=run_config,
+            mode=mode,
+            preset=preset,
+            tags=tags,
+            solid_selection=solid_selection,
+            raise_on_error=raise_on_error,
+        )
 
 
 @telemetry_wrapper
 def _logged_execute_pipeline(
     pipeline,
+    instance,
     run_config=None,
     mode=None,
     preset=None,
     tags=None,
     solid_selection=None,
-    instance=None,
     raise_on_error=True,
 ):
-
+    check.inst_param(instance, "instance", DagsterInstance)
     (
         pipeline,
         run_config,
-        instance,
         mode,
         tags,
         solids_to_execute,
@@ -334,7 +346,6 @@ def _logged_execute_pipeline(
         preset=preset,
         tags=tags,
         solid_selection=solid_selection,
-        instance=instance,
     )
 
     log_repo_stats(instance=instance, pipeline=pipeline, source="execute_pipeline")
@@ -355,12 +366,13 @@ def reexecute_pipeline(
     pipeline,
     parent_run_id,
     run_config=None,
-    step_keys_to_execute=None,
+    step_selection=None,
     mode=None,
     preset=None,
     tags=None,
     instance=None,
     raise_on_error=True,
+    step_keys_to_execute=None,
 ):
     """Reexecute an existing pipeline run.
 
@@ -373,7 +385,16 @@ def reexecute_pipeline(
             instance.
         run_config (Optional[dict]): The environment configuration that parametrizes this run,
             as a dict.
-        step_keys_to_execute (Optional[List[str]]): Keys of the steps to execute.
+        step_selection (Optional[List[str]]): A list of step selection queries (including single
+            step keys) to execute. For example:
+            - ['some_solid.compute']: select the execution step "some_solid.compute" itself.
+            - ['*some_solid.compute']: select the step "some_solid.compute" and all its ancestors
+                (upstream dependencies).
+            - ['*some_solid.compute+++']: select the step "some_solid.compute", all its ancestors,
+                and its descendants (downstream dependencies) within 3 levels down.
+            - ['*some_solid.compute', 'other_solid_a.compute', 'other_solid_b.compute+']: select
+                "some_solid.compute" and all its ancestors, "other_solid_a.compute" itself, and
+                "other_solid_b.compute" and its direct child execution steps.
         mode (Optional[str]): The name of the pipeline mode to use. You may not set both ``mode``
             and ``preset``.
         preset (Optional[str]): The name of the pipeline preset to use. You may not set both
@@ -390,49 +411,64 @@ def reexecute_pipeline(
 
     For the asynchronous version, see :py:func:`reexecute_pipeline_iterator`.
     """
+
+    step_selection = canonicalize_backcompat_args(
+        step_selection, "step_selection", step_keys_to_execute, "step_keys_to_execute", "0.10.0",
+    )
+    check.opt_list_param(step_selection, "step_selection", of_type=str)
+
     check.str_param(parent_run_id, "parent_run_id")
 
-    (pipeline, run_config, instance, mode, tags, _, _) = _check_execute_pipeline_args(
-        pipeline=pipeline,
-        run_config=run_config,
-        mode=mode,
-        preset=preset,
-        tags=tags,
-        instance=instance,
-    )
+    with _ephemeral_instance_if_missing(instance) as execute_instance:
+        (pipeline, run_config, mode, tags, _, _) = _check_execute_pipeline_args(
+            pipeline=pipeline, run_config=run_config, mode=mode, preset=preset, tags=tags,
+        )
 
-    parent_pipeline_run = instance.get_run_by_id(parent_run_id)
-    check.invariant(
-        parent_pipeline_run,
-        "No parent run with id {parent_run_id} found in instance.".format(
-            parent_run_id=parent_run_id
-        ),
-    )
+        parent_pipeline_run = execute_instance.get_run_by_id(parent_run_id)
+        check.invariant(
+            parent_pipeline_run,
+            "No parent run with id {parent_run_id} found in instance.".format(
+                parent_run_id=parent_run_id
+            ),
+        )
 
-    pipeline_run = instance.create_run_for_pipeline(
-        pipeline_def=pipeline.get_definition(),
-        run_config=run_config,
-        mode=mode,
-        tags=tags,
-        solid_selection=parent_pipeline_run.solid_selection,
-        solids_to_execute=parent_pipeline_run.solids_to_execute,
-        step_keys_to_execute=step_keys_to_execute,
-        root_run_id=parent_pipeline_run.root_run_id or parent_pipeline_run.run_id,
-        parent_run_id=parent_pipeline_run.run_id,
-    )
+        # resolve step selection DSL queries using parent execution plan snapshot
+        if step_selection:
+            parent_execution_plan_snapshot = instance.get_execution_plan_snapshot(
+                parent_pipeline_run.execution_plan_snapshot_id
+            )
+            step_keys_to_execute = parse_step_selection(
+                parent_execution_plan_snapshot.step_deps, step_selection
+            )
+        else:
+            step_keys_to_execute = None
 
-    return execute_run(pipeline, pipeline_run, instance, raise_on_error=raise_on_error)
+        pipeline_run = execute_instance.create_run_for_pipeline(
+            pipeline_def=pipeline.get_definition(),
+            run_config=run_config,
+            mode=mode,
+            tags=tags,
+            solid_selection=parent_pipeline_run.solid_selection,
+            solids_to_execute=parent_pipeline_run.solids_to_execute,
+            # convert to frozenset https://github.com/dagster-io/dagster/issues/2914
+            step_keys_to_execute=list(step_keys_to_execute) if step_keys_to_execute else None,
+            root_run_id=parent_pipeline_run.root_run_id or parent_pipeline_run.run_id,
+            parent_run_id=parent_pipeline_run.run_id,
+        )
+
+        return execute_run(pipeline, pipeline_run, execute_instance, raise_on_error=raise_on_error)
 
 
 def reexecute_pipeline_iterator(
     pipeline,
     parent_run_id,
     run_config=None,
-    step_keys_to_execute=None,
+    step_selection=None,
     mode=None,
     preset=None,
     tags=None,
     instance=None,
+    step_keys_to_execute=None,
 ):
     """Reexecute a pipeline iteratively.
 
@@ -449,7 +485,16 @@ def reexecute_pipeline_iterator(
             instance.
         run_config (Optional[dict]): The environment configuration that parametrizes this run,
             as a dict.
-        step_keys_to_execute (Optional[List[str]]): Keys of the steps to execute.
+        step_selection (Optional[List[str]]): A list of step selection queries (including single
+            step keys) to execute. For example:
+            - ['some_solid.compute']: select the execution step "some_solid.compute" itself.
+            - ['*some_solid.compute']: select the step "some_solid.compute" and all its ancestors
+                (upstream dependencies).
+            - ['*some_solid.compute+++']: select the step "some_solid.compute", all its ancestors,
+                and its descendants (downstream dependencies) within 3 levels down.
+            - ['*some_solid.compute', 'other_solid_a.compute', 'other_solid_b.compute+']: select
+                "some_solid.compute" and all its ancestors, "other_solid_a.compute" itself, and
+                "other_solid_b.compute" and its direct child execution steps.
         mode (Optional[str]): The name of the pipeline mode to use. You may not set both ``mode``
             and ``preset``.
         preset (Optional[str]): The name of the pipeline preset to use. You may not set both
@@ -462,38 +507,56 @@ def reexecute_pipeline_iterator(
     Returns:
       Iterator[DagsterEvent]: The stream of events resulting from pipeline reexecution.
     """
+
+    step_selection = canonicalize_backcompat_args(
+        step_selection, "step_selection", step_keys_to_execute, "step_keys_to_execute", "0.10.0",
+    )
+    check.opt_list_param(step_selection, "step_selection", of_type=str)
+
     check.str_param(parent_run_id, "parent_run_id")
 
-    (pipeline, run_config, instance, mode, tags, _, _) = _check_execute_pipeline_args(
-        pipeline=pipeline,
-        run_config=run_config,
-        mode=mode,
-        preset=preset,
-        tags=tags,
-        instance=instance,
-        solid_selection=None,
-    )
-    parent_pipeline_run = instance.get_run_by_id(parent_run_id)
-    check.invariant(
-        parent_pipeline_run,
-        "No parent run with id {parent_run_id} found in instance.".format(
-            parent_run_id=parent_run_id
-        ),
-    )
+    with _ephemeral_instance_if_missing(instance) as execute_instance:
+        (pipeline, run_config, mode, tags, _, _) = _check_execute_pipeline_args(
+            pipeline=pipeline,
+            run_config=run_config,
+            mode=mode,
+            preset=preset,
+            tags=tags,
+            solid_selection=None,
+        )
+        parent_pipeline_run = execute_instance.get_run_by_id(parent_run_id)
+        check.invariant(
+            parent_pipeline_run,
+            "No parent run with id {parent_run_id} found in instance.".format(
+                parent_run_id=parent_run_id
+            ),
+        )
 
-    pipeline_run = instance.create_run_for_pipeline(
-        pipeline_def=pipeline.get_definition(),
-        run_config=run_config,
-        mode=mode,
-        tags=tags,
-        solid_selection=parent_pipeline_run.solid_selection,
-        solids_to_execute=parent_pipeline_run.solids_to_execute,
-        step_keys_to_execute=step_keys_to_execute,
-        root_run_id=parent_pipeline_run.root_run_id or parent_pipeline_run.run_id,
-        parent_run_id=parent_pipeline_run.run_id,
-    )
+        # resolve step selection DSL queries using parent execution plan snapshot
+        if step_selection:
+            parent_execution_plan_snapshot = instance.get_execution_plan_snapshot(
+                parent_pipeline_run.execution_plan_snapshot_id
+            )
+            step_keys_to_execute = parse_step_selection(
+                parent_execution_plan_snapshot.step_deps, step_selection
+            )
+        else:
+            step_keys_to_execute = None
 
-    return execute_run_iterator(pipeline, pipeline_run, instance)
+        pipeline_run = execute_instance.create_run_for_pipeline(
+            pipeline_def=pipeline.get_definition(),
+            run_config=run_config,
+            mode=mode,
+            tags=tags,
+            solid_selection=parent_pipeline_run.solid_selection,
+            solids_to_execute=parent_pipeline_run.solids_to_execute,
+            # convert to frozenset https://github.com/dagster-io/dagster/issues/2914
+            step_keys_to_execute=list(step_keys_to_execute) if step_keys_to_execute else None,
+            root_run_id=parent_pipeline_run.root_run_id or parent_pipeline_run.run_id,
+            parent_run_id=parent_pipeline_run.run_id,
+        )
+
+        return execute_run_iterator(pipeline, pipeline_run, execute_instance)
 
 
 def execute_plan_iterator(
@@ -671,9 +734,7 @@ class _ExecuteRunWithPlanIterable(object):
                     yield event
 
 
-def _check_execute_pipeline_args(
-    pipeline, run_config, mode, preset, tags, instance, solid_selection=None
-):
+def _check_execute_pipeline_args(pipeline, run_config, mode, preset, tags, solid_selection=None):
     pipeline = _check_pipeline(pipeline)
     pipeline_def = pipeline.get_definition()
     check.inst_param(pipeline_def, "pipeline_def", PipelineDefinition)
@@ -749,9 +810,6 @@ def _check_execute_pipeline_args(
 
     tags = merge_dicts(pipeline_def.tags, tags)
 
-    check.opt_inst_param(instance, "instance", DagsterInstance)
-    instance = instance or DagsterInstance.ephemeral()
-
     # generate pipeline subset from the given solid_selection
     if solid_selection:
         pipeline = pipeline.subset_for_execution(solid_selection)
@@ -759,7 +817,6 @@ def _check_execute_pipeline_args(
     return (
         pipeline,
         run_config,
-        instance,
         mode,
         tags,
         pipeline.solids_to_execute,
