@@ -6,7 +6,7 @@ import six
 class Executor(six.with_metaclass(abc.ABCMeta)):  # pylint: disable=no-init
     @abc.abstractmethod
     def execute(self, pipeline_context, execution_plan):
-        '''
+        """
         For the given context and execution plan, orchestrate a series of sub plan executions in a way that satisfies the whole plan being executed.
 
         Args:
@@ -15,4 +15,13 @@ class Executor(six.with_metaclass(abc.ABCMeta)):  # pylint: disable=no-init
 
         Returns:
             A stream of dagster events.
-        '''
+        """
+
+    @abc.abstractproperty
+    def retries(self):
+        """
+        The Retries state / policy for this instance of the Executor. Executors should allow this to be
+        controlled via configuration if possible.
+
+        Returns: Retries
+        """
