@@ -18,29 +18,29 @@ from .config_schema import dagster_type_loader, dagster_type_materializer
 
 
 def define_typed_input_schema_dict(value_config_type):
-    check.inst_param(value_config_type, 'value_config_type', ConfigType)
+    check.inst_param(value_config_type, "value_config_type", ConfigType)
     return Selector(
         {
-            'value': Field(value_config_type),
-            'json': define_path_dict_field(),
-            'pickle': define_path_dict_field(),
+            "value": Field(value_config_type),
+            "json": define_path_dict_field(),
+            "pickle": define_path_dict_field(),
         },
     )
 
 
 def load_type_input_schema_dict(value):
     file_type, file_options = list(value.items())[0]
-    if file_type == 'value':
+    if file_type == "value":
         return file_options
-    elif file_type == 'json':
-        with open(file_options['path'], 'r') as ff:
+    elif file_type == "json":
+        with open(file_options["path"], "r") as ff:
             value_dict = seven.json.load(ff)
-            return value_dict['value']
-    elif file_type == 'pickle':
-        with open(file_options['path'], 'rb') as ff:
+            return value_dict["value"]
+    elif file_type == "pickle":
+        with open(file_options["path"], "rb") as ff:
             return pickle.load(ff)
     else:
-        check.failed('Unsupported key {key}'.format(key=file_type))
+        check.failed("Unsupported key {key}".format(key=file_type))
 
 
 def define_any_input_schema():
@@ -52,9 +52,9 @@ def define_any_input_schema():
 
 
 def define_builtin_scalar_input_schema(scalar_name, config_scalar_type):
-    check.str_param(scalar_name, 'scalar_name')
-    check.inst_param(config_scalar_type, 'config_scalar_type', ConfigType)
-    check.param_invariant(config_scalar_type.kind == ConfigTypeKind.SCALAR, 'config_scalar_type')
+    check.str_param(scalar_name, "scalar_name")
+    check.inst_param(config_scalar_type, "config_scalar_type", ConfigType)
+    check.param_invariant(config_scalar_type.kind == ConfigTypeKind.SCALAR, "config_scalar_type")
 
     @dagster_type_loader(
         ScalarUnion(
@@ -69,13 +69,13 @@ def define_builtin_scalar_input_schema(scalar_name, config_scalar_type):
 
 
 def define_path_dict_field():
-    return {'path': Field(ConfigStringInstance)}
+    return {"path": Field(ConfigStringInstance)}
 
 
 def define_builtin_scalar_output_schema(scalar_name):
-    check.str_param(scalar_name, 'scalar_name')
+    check.str_param(scalar_name, "scalar_name")
 
-    schema_cls = Selector({'json': define_path_dict_field(), 'pickle': define_path_dict_field()})
+    schema_cls = Selector({"json": define_path_dict_field(), "pickle": define_path_dict_field()})
 
     @dagster_type_materializer(schema_cls)
     def _buildint_materializer(_context, config_value, runtime_value):
@@ -83,35 +83,35 @@ def define_builtin_scalar_output_schema(scalar_name):
 
         file_type, file_options = list(config_value.items())[0]
 
-        if file_type == 'json':
-            json_file_path = file_options['path']
-            json_value = seven.json.dumps({'value': runtime_value})
-            with open(json_file_path, 'w') as ff:
+        if file_type == "json":
+            json_file_path = file_options["path"]
+            json_value = seven.json.dumps({"value": runtime_value})
+            with open(json_file_path, "w") as ff:
                 ff.write(json_value)
             return AssetMaterialization.file(json_file_path)
-        elif file_type == 'pickle':
-            pickle_file_path = file_options['path']
-            with open(pickle_file_path, 'wb') as ff:
+        elif file_type == "pickle":
+            pickle_file_path = file_options["path"]
+            with open(pickle_file_path, "wb") as ff:
                 pickle.dump(runtime_value, ff)
             return AssetMaterialization.file(pickle_file_path)
         else:
-            check.failed('Unsupported file type: {file_type}'.format(file_type=file_type))
+            check.failed("Unsupported file type: {file_type}".format(file_type=file_type))
 
     return _buildint_materializer
 
 
 class BuiltinSchemas(object):
     ANY_INPUT = define_any_input_schema()
-    ANY_OUTPUT = define_builtin_scalar_output_schema('Any')
+    ANY_OUTPUT = define_builtin_scalar_output_schema("Any")
 
-    BOOL_INPUT = define_builtin_scalar_input_schema('Bool', ConfigBoolInstance)
-    BOOL_OUTPUT = define_builtin_scalar_output_schema('Bool')
+    BOOL_INPUT = define_builtin_scalar_input_schema("Bool", ConfigBoolInstance)
+    BOOL_OUTPUT = define_builtin_scalar_output_schema("Bool")
 
-    FLOAT_INPUT = define_builtin_scalar_input_schema('Float', ConfigFloatInstance)
-    FLOAT_OUTPUT = define_builtin_scalar_output_schema('Float')
+    FLOAT_INPUT = define_builtin_scalar_input_schema("Float", ConfigFloatInstance)
+    FLOAT_OUTPUT = define_builtin_scalar_output_schema("Float")
 
-    INT_INPUT = define_builtin_scalar_input_schema('Int', ConfigIntInstance)
-    INT_OUTPUT = define_builtin_scalar_output_schema('Int')
+    INT_INPUT = define_builtin_scalar_input_schema("Int", ConfigIntInstance)
+    INT_OUTPUT = define_builtin_scalar_output_schema("Int")
 
-    STRING_INPUT = define_builtin_scalar_input_schema('String', ConfigStringInstance)
-    STRING_OUTPUT = define_builtin_scalar_output_schema('String')
+    STRING_INPUT = define_builtin_scalar_input_schema("String", ConfigStringInstance)
+    STRING_OUTPUT = define_builtin_scalar_output_schema("String")
