@@ -27,12 +27,12 @@ def is_supported_runtime_python_builtin(ttype):
 
 
 def remap_python_builtin_for_runtime(ttype):
-    '''This function remaps a python type to a Dagster type, or passes it through if it cannot be
+    """This function remaps a python type to a Dagster type, or passes it through if it cannot be
     remapped.
-    '''
+    """
     from dagster.core.types.dagster_type import resolve_dagster_type
 
-    check.param_invariant(is_supported_runtime_python_builtin(ttype), 'ttype')
+    check.param_invariant(is_supported_runtime_python_builtin(ttype), "ttype")
 
     return resolve_dagster_type(SUPPORTED_RUNTIME_BUILTINS[ttype])
 
@@ -52,11 +52,11 @@ def is_supported_config_python_builtin(ttype):
 
 
 def remap_python_builtin_for_config(ttype):
-    '''This function remaps a python type to a Dagster type, or passes it through if it cannot be
+    """This function remaps a python type to a Dagster type, or passes it through if it cannot be
     remapped.
-    '''
+    """
     from dagster.config.field import resolve_to_config_type
 
-    check.param_invariant(is_supported_config_python_builtin(ttype), 'ttype')
+    check.param_invariant(is_supported_config_python_builtin(ttype), "ttype")
 
     return resolve_to_config_type(SUPPORTED_CONFIG_BUILTINS[ttype])
